"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var fields_tree_1 = require("./fields-tree");
var graphql_codegen_visitor_plugin_common_1 = require("graphql-codegen-visitor-plugin-common");
var graphql_codegen_typescript_1 = require("graphql-codegen-typescript");
var autoBind = require("auto-bind");
var index_1 = require("./index");
var graphql_1 = require("graphql");
var TsMongoVisitor = /** @class */ (function (_super) {
    __extends(TsMongoVisitor, _super);
    function TsMongoVisitor(_schema, pluginConfig) {
        var _this = _super.call(this, pluginConfig, {
            dbTypeSuffix: pluginConfig.dbTypeSuffix || 'DbObject',
            dbInterfaceSuffix: pluginConfig.dbInterfaceSuffix || 'DbInterface',
            objectIdType: pluginConfig.objectIdType || 'ObjectID',
            idFieldName: pluginConfig.idFieldName || '_id',
            enumsAsString: graphql_codegen_visitor_plugin_common_1.getConfigValue(pluginConfig.enumsAsString, true),
            avoidOptionals: graphql_codegen_visitor_plugin_common_1.getConfigValue(pluginConfig.avoidOptionals, false)
        }) || this;
        _this._schema = _schema;
        autoBind(_this);
        _this._variablesTransformer = new graphql_codegen_typescript_1.TypeScriptOperationVariablesToObject(_this.scalars, _this.convertName, false, false);
        return _this;
    }
    TsMongoVisitor.prototype._resolveDirectiveValue = function (valueNode) {
        var _this = this;
        switch (valueNode.kind) {
            case graphql_1.Kind.INT:
            case graphql_1.Kind.STRING:
            case graphql_1.Kind.FLOAT:
            case graphql_1.Kind.BOOLEAN:
            case graphql_1.Kind.ENUM:
                return valueNode.value;
            case graphql_1.Kind.LIST:
                return valueNode.values.map(function (v) { return _this._resolveDirectiveValue(v); });
            case graphql_1.Kind.NULL:
                return null;
            case graphql_1.Kind.OBJECT:
                return valueNode.fields.reduce(function (prev, f) {
                    var _a;
                    return __assign({}, prev, (_a = {}, _a[f.name.value] = _this._resolveDirectiveValue(f.value), _a));
                }, {});
            default:
                return undefined;
        }
    };
    TsMongoVisitor.prototype._getDirectiveArgValue = function (node, argName) {
        if (!node || !node.arguments || node.arguments.length === 0) {
            return undefined;
        }
        var foundArgument = node.arguments.find(function (a) { return a.name.value === argName; });
        if (!foundArgument) {
            return undefined;
        }
        return this._resolveDirectiveValue(foundArgument.value);
    };
    TsMongoVisitor.prototype._getDirectiveFromAstNode = function (node, directiveName) {
        if (!node || !node.directives || node.directives.length === 0) {
            return null;
        }
        var foundDirective = node.directives.find(function (d) { return d.name === directiveName || (d.name.value && d.name.value === directiveName); });
        if (!foundDirective) {
            return null;
        }
        return foundDirective;
    };
    TsMongoVisitor.prototype._buildInterfaces = function (interfaces) {
        var _this = this;
        return (interfaces || [])
            .map(function (namedType) {
            var schemaType = _this._schema.getType(namedType.name.value);
            var abstractEntityDirective = _this._getDirectiveFromAstNode(schemaType.astNode, index_1.Directives.ABSTRACT_ENTITY);
            if (!abstractEntityDirective) {
                return null;
            }
            return _this.convertName(namedType.name.value, { suffix: _this.config.dbInterfaceSuffix });
        })
            .filter(function (a) { return a; });
    };
    TsMongoVisitor.prototype._handleIdField = function (fieldNode, tree, addOptionalSign) {
        tree.addField("" + this.config.idFieldName + (addOptionalSign ? '?' : ''), this._variablesTransformer.wrapAstTypeWithModifiers(this.config.objectIdType, fieldNode.type));
    };
    TsMongoVisitor.prototype._handleLinkField = function (fieldNode, tree, mapPath, addOptionalSign) {
        var coreType = graphql_codegen_visitor_plugin_common_1.getBaseTypeNode(fieldNode.type);
        var type = this.convertName(coreType, { suffix: this.config.dbTypeSuffix });
        tree.addField(mapPath ? mapPath : "" + fieldNode.name.value + (addOptionalSign ? '?' : ''), this._variablesTransformer.wrapAstTypeWithModifiers(type + "['" + this.config.idFieldName + "']", fieldNode.type));
    };
    TsMongoVisitor.prototype._handleColumnField = function (fieldNode, tree, columnDirective, mapPath, addOptionalSign) {
        var overrideType = this._getDirectiveArgValue(columnDirective, 'overrideType');
        var coreType = graphql_codegen_visitor_plugin_common_1.getBaseTypeNode(fieldNode.type);
        var type = null;
        if (this.scalars[coreType.name.value]) {
            type = this.scalars[coreType.name.value];
        }
        else {
            var schemaType = this._schema.getType(coreType.name.value);
            if (graphql_1.isEnumType(schemaType) && this.config.enumsAsString) {
                type = this.scalars.String;
            }
            else {
                type = coreType.name.value;
            }
        }
        tree.addField(mapPath ? mapPath : "" + fieldNode.name.value + (addOptionalSign ? '?' : ''), overrideType ? overrideType : this._variablesTransformer.wrapAstTypeWithModifiers(type, fieldNode.type));
    };
    TsMongoVisitor.prototype._handleEmbeddedField = function (fieldNode, tree, mapPath, addOptionalSign) {
        var coreType = graphql_codegen_visitor_plugin_common_1.getBaseTypeNode(fieldNode.type);
        var type = this.convertName(coreType, { suffix: this.config.dbTypeSuffix });
        tree.addField(mapPath ? mapPath : "" + fieldNode.name.value + (addOptionalSign ? '?' : ''), this._variablesTransformer.wrapAstTypeWithModifiers(type, fieldNode.type));
    };
    TsMongoVisitor.prototype._buildFieldsTree = function (fields) {
        var _this = this;
        var tree = new fields_tree_1.FieldsTree();
        fields.forEach(function (field) {
            var idDirective = _this._getDirectiveFromAstNode(field, index_1.Directives.ID);
            var linkDirective = _this._getDirectiveFromAstNode(field, index_1.Directives.LINK);
            var columnDirective = _this._getDirectiveFromAstNode(field, index_1.Directives.COLUMN);
            var embeddedDirective = _this._getDirectiveFromAstNode(field, index_1.Directives.EMBEDDED);
            var mapDirective = _this._getDirectiveFromAstNode(field, index_1.Directives.MAP);
            var mapPath = _this._getDirectiveArgValue(mapDirective, 'path');
            var addOptionalSign = !_this.config.avoidOptionals && field.type.kind !== graphql_1.Kind.NON_NULL_TYPE;
            if (idDirective) {
                _this._handleIdField(field, tree, addOptionalSign);
            }
            else if (linkDirective) {
                _this._handleLinkField(field, tree, mapPath, addOptionalSign);
            }
            else if (columnDirective) {
                _this._handleColumnField(field, tree, columnDirective, mapPath, addOptionalSign);
            }
            else if (embeddedDirective) {
                _this._handleEmbeddedField(field, tree, mapPath, addOptionalSign);
            }
        });
        return tree;
    };
    TsMongoVisitor.prototype._addAdditionalFields = function (tree, additioalFields) {
        if (!additioalFields || additioalFields.length === 0) {
            return;
        }
        for (var _i = 0, additioalFields_1 = additioalFields; _i < additioalFields_1.length; _i++) {
            var field = additioalFields_1[_i];
            tree.addField(field.path, field.type);
        }
    };
    TsMongoVisitor.prototype.InterfaceTypeDefinition = function (node) {
        var abstractEntityDirective = this._getDirectiveFromAstNode(node, index_1.Directives.ABSTRACT_ENTITY);
        if (abstractEntityDirective === null) {
            return null;
        }
        var discriminatorField = this._getDirectiveArgValue(abstractEntityDirective, 'discriminatorField');
        var additionalFields = this._getDirectiveArgValue(abstractEntityDirective, 'additionalFields');
        var fields = this._buildFieldsTree(node.fields);
        fields.addField(discriminatorField, this.scalars.String);
        this._addAdditionalFields(fields, additionalFields);
        return new graphql_codegen_visitor_plugin_common_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(node, { suffix: this.config.dbInterfaceSuffix }))
            .withBlock(fields.string).string;
    };
    TsMongoVisitor.prototype.UnionTypeDefinition = function (node) {
        var _this = this;
        var unionDirective = this._getDirectiveFromAstNode(node, index_1.Directives.UNION);
        if (unionDirective === null) {
            return null;
        }
        var discriminatorField = this._getDirectiveArgValue(unionDirective, 'discriminatorField');
        var possibleTypes = node.types
            .map(function (namedType) {
            var schemaType = _this._schema.getType(namedType.name.value);
            var entityDirective = _this._getDirectiveFromAstNode(schemaType.astNode, index_1.Directives.ENTITY);
            var abstractEntityDirective = _this._getDirectiveFromAstNode(schemaType.astNode, index_1.Directives.ABSTRACT_ENTITY);
            if (entityDirective) {
                return _this.convertName(namedType, { suffix: _this.config.dbTypeSuffix });
            }
            else if (abstractEntityDirective) {
                return _this.convertName(namedType, { suffix: _this.config.dbInterfaceSuffix });
            }
            return null;
        })
            .filter(function (a) { return a; });
        if (possibleTypes.length === 0) {
            return null;
        }
        var additionalFields = this._getDirectiveArgValue(unionDirective, 'additionalFields');
        var fields = new fields_tree_1.FieldsTree();
        fields.addField(discriminatorField, this.scalars.String);
        this._addAdditionalFields(fields, additionalFields);
        return new graphql_codegen_visitor_plugin_common_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(node, { suffix: this.config.dbTypeSuffix }))
            .withContent("(" + possibleTypes.join(' | ') + ") & ")
            .withBlock(fields.string).string;
    };
    TsMongoVisitor.prototype.ObjectTypeDefinition = function (node) {
        var entityDirective = this._getDirectiveFromAstNode(node, index_1.Directives.ENTITY);
        if (entityDirective === null) {
            return null;
        }
        var implementingInterfaces = this._buildInterfaces(node.interfaces);
        var fields = this._buildFieldsTree(node.fields);
        var additionalFields = this._getDirectiveArgValue(entityDirective, 'additionalFields');
        this._addAdditionalFields(fields, additionalFields);
        return new graphql_codegen_visitor_plugin_common_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(node, { suffix: this.config.dbTypeSuffix }))
            .withContent(implementingInterfaces.length ? implementingInterfaces.join(' & ') + ' & ' : '')
            .withBlock(fields.string).string;
    };
    return TsMongoVisitor;
}(graphql_codegen_visitor_plugin_common_1.BaseVisitor));
exports.TsMongoVisitor = TsMongoVisitor;
//# sourceMappingURL=visitor.js.map